/*
 * Copyright (c) 2017 SpaceToad and the BuildCraft team
 * This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy of the MPL was not
 * distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/
 */

package buildcraft.builders.client.render;

import java.util.Collections;

import javax.vecmath.Point3f;

import net.minecraft.client.renderer.BufferBuilder;
import net.minecraft.item.ItemStack;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.Vec3d;
import net.minecraft.world.World;

import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import buildcraft.lib.client.model.ModelUtil;
import buildcraft.lib.client.render.ItemRenderUtil;
import buildcraft.lib.client.render.laser.LaserData_BC8;
import buildcraft.lib.client.render.laser.LaserRenderer_BC8;
import buildcraft.lib.misc.MathUtil;
import buildcraft.lib.misc.VecUtil;

import buildcraft.builders.BCBuildersSprites;
import buildcraft.builders.snapshot.ITileForSnapshotBuilder;
import buildcraft.builders.snapshot.SnapshotBuilder;
import buildcraft.core.client.BuildCraftLaserManager;

import buildcraft.builders.snapshot.SnapshotBuilder.PlaceTask;

@SideOnly(Side.CLIENT)
public class RenderSnapshotBuilder {
    public static <T extends ITileForSnapshotBuilder> void render(
            SnapshotBuilder<T> snapshotBuilder,
            World world,
            BlockPos tilePos,
            double x,
            double y,
            double z,
            float partialTicks,
            BufferBuilder bb
    ) {
        for (SnapshotBuilder<T>.PlaceTask placeTask : snapshotBuilder.clientPlaceTasks) {
            Vec3d prevPos = snapshotBuilder.prevClientPlaceTasks.stream()
                .filter(renderTaskLocal -> renderTaskLocal.pos.equals(placeTask.pos))
                .map(snapshotBuilder::getPlaceTaskItemPos)
                .findFirst()
                .orElse(snapshotBuilder.getPlaceTaskItemPos(snapshotBuilder.new PlaceTask(tilePos, Collections.emptyList(), 0L)));
            Vec3d pos = prevPos.func_178787_e(snapshotBuilder.getPlaceTaskItemPos(placeTask).func_178788_d(prevPos).func_186678_a(partialTicks));
            for (ItemStack item : placeTask.items) {
                ItemRenderUtil.renderItemStack(
                    x - tilePos.func_177958_n() + pos.field_72450_a,
                    y - tilePos.func_177956_o() + pos.field_72448_b,
                    z - tilePos.func_177952_p() + pos.field_72449_c,
                    item,
                    world.func_175626_b(new BlockPos(pos), 0),
                    EnumFacing.SOUTH,
                    bb
                );
            }
            ItemRenderUtil.endItemBatch();
        }

        Vec3d robotPos = snapshotBuilder.robotPos;
        if (robotPos != null) {
            if (snapshotBuilder.prevRobotPos != null) {
                robotPos = snapshotBuilder.prevRobotPos.func_178787_e(robotPos.func_178788_d(snapshotBuilder.prevRobotPos).func_186678_a(partialTicks));
            }

            bb.func_178969_c(x - tilePos.func_177958_n(), y - tilePos.func_177956_o(), z - tilePos.func_177952_p());

            int i = 0;
            for (EnumFacing face : EnumFacing.field_82609_l) {
                ModelUtil.createFace(
                    face,
                    new Point3f((float) robotPos.field_72450_a, (float) robotPos.field_72448_b, (float) robotPos.field_72449_c),
                    new Point3f(4 / 16F, 4 / 16F, 4 / 16F),
                    new ModelUtil.UvFaceData(
                        BCBuildersSprites.ROBOT.getInterpU((i * 8) / 64D),
                        BCBuildersSprites.ROBOT.getInterpV(0 / 64D),
                        BCBuildersSprites.ROBOT.getInterpU(((i + 1) * 8) / 64D),
                        BCBuildersSprites.ROBOT.getInterpV(8 / 64D)
                    )
                )
                    .lighti(world.func_175626_b(new BlockPos(robotPos), 0))
                    .render(bb);
                i++;
            }

            for (SnapshotBuilder.BreakTask breakTask : snapshotBuilder.clientBreakTasks) {
                LaserRenderer_BC8.renderLaserDynamic(
                    new LaserData_BC8(
                        BuildCraftLaserManager.POWERS[(int) Math.round(
                            MathUtil.clamp(
                                breakTask.power * 1D / breakTask.getTarget(),
                                0D,
                                1D
                            ) * (BuildCraftLaserManager.POWERS.length - 1)
                        )],
                        robotPos.func_178788_d(new Vec3d(0, 0.27, 0)),
                        new Vec3d(breakTask.pos).func_178787_e(VecUtil.VEC_HALF),
                        1 / 16D
                    ),
                    bb
                );
            }
        }

        bb.func_178969_c(0, 0, 0);
    }
}
