/*
 * Copyright (c) 2017 SpaceToad and the BuildCraft team
 * This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy of the MPL was not
 * distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/
 */

package buildcraft.factory.tile;

import javax.annotation.Nonnull;

import net.minecraft.block.state.IBlockState;
import net.minecraft.init.Items;
import net.minecraft.item.ItemStack;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IWorldEventListener;
import net.minecraft.world.World;
import net.minecraft.world.WorldServer;

import net.minecraftforge.fluids.Fluid;

import buildcraft.api.core.EnumPipePart;
import buildcraft.api.core.SafeTimeTracker;
import buildcraft.api.mj.IMjReceiver;

import buildcraft.lib.inventory.AutomaticProvidingTransactor;
import buildcraft.lib.misc.BlockUtil;
import buildcraft.lib.misc.CapUtil;
import buildcraft.lib.misc.InventoryUtil;
import buildcraft.lib.mj.MjBatteryReceiver;
import buildcraft.lib.world.WorldEventListenerAdapter;

import buildcraft.core.BCCoreConfig;
import buildcraft.factory.BCFactoryBlocks;

public class TileMiningWell extends TileMiner {
    private boolean shouldCheck = true;
    private final SafeTimeTracker tracker = new SafeTimeTracker(256);
    private final IWorldEventListener worldEventListener = new WorldEventListenerAdapter() {
        @Override
        public void func_184376_a(@Nonnull World world,
                                      @Nonnull BlockPos pos,
                                      @Nonnull IBlockState oldState,
                                      @Nonnull IBlockState newState,
                                      int flags) {
            if (pos.func_177958_n() == TileMiningWell.this.field_174879_c.func_177958_n() &&
                pos.func_177956_o() <= TileMiningWell.this.field_174879_c.func_177956_o() &&
                pos.func_177952_p() == TileMiningWell.this.field_174879_c.func_177952_p()) {
                shouldCheck = true;
            }
        }
    };

    public TileMiningWell() {
        super();
        caps.addCapabilityInstance(CapUtil.CAP_ITEM_TRANSACTOR, AutomaticProvidingTransactor.INSTANCE, EnumPipePart.VALUES);
    }

    @Override
    protected void mine() {
        if (currentPos != null && canBreak()) {
            shouldCheck = true;
            long target = BlockUtil.computeBlockBreakPower(field_145850_b, currentPos);
            progress += battery.extractPower(0, target - progress);
            if (progress >= target) {
                progress = 0;
                field_145850_b.func_175715_c(currentPos.hashCode(), currentPos, -1);
                BlockUtil.breakBlockAndGetDrops(
                    (WorldServer) field_145850_b,
                    currentPos,
                    new ItemStack(Items.field_151046_w),
                    getOwner()
                ).ifPresent(stacks ->
                    stacks.forEach(stack -> InventoryUtil.addToBestAcceptor(field_145850_b, field_174879_c, null, stack))
                );
                nextPos();
            } else {
                if (!field_145850_b.func_175623_d(currentPos)) {
                    field_145850_b.func_175715_c(currentPos.hashCode(), currentPos, (int) ((progress * 9) / target));
                }
            }
        } else if (shouldCheck || tracker.markTimeIfDelay(field_145850_b)) {
            nextPos();
            if (currentPos == null) {
                shouldCheck = false;
            }
        }
    }

    private boolean canBreak() {
        if (field_145850_b.func_175623_d(currentPos) || BlockUtil.isUnbreakableBlock(field_145850_b, currentPos, getOwner())) {
            return false;
        }

        Fluid fluid = BlockUtil.getFluidWithFlowing(field_145850_b, currentPos);
        return fluid == null || fluid.getViscosity() <= 1000;
    }

    private void nextPos() {
        currentPos = field_174879_c;
        while (true) {
            currentPos = currentPos.func_177977_b();
            if (field_145850_b.func_189509_E(currentPos)) {
                break;
            }
            if (field_174879_c.func_177956_o() - currentPos.func_177956_o() > BCCoreConfig.miningMaxDepth) {
                break;
            }
            if (canBreak()) {
                updateLength();
                return;
            } else if (!field_145850_b.func_175623_d(currentPos) && field_145850_b.func_180495_p(currentPos).func_177230_c() != BCFactoryBlocks.tube) {
                break;
            }
        }
        currentPos = null;
        updateLength();
    }

    @Override
    public void func_145829_t() {
        super.func_145829_t();
        if (!field_145850_b.field_72995_K) {
            field_145850_b.func_72954_a(worldEventListener);
        }
    }

    @Override
    public void func_145843_s() {
        super.func_145843_s();
        if (!field_145850_b.field_72995_K) {
            field_145850_b.func_72848_b(worldEventListener);
            if (currentPos != null) {
                field_145850_b.func_175715_c(currentPos.hashCode(), currentPos, -1);
            }
        }
    }

    @Override
    protected IMjReceiver createMjReceiver() {
        return new MjBatteryReceiver(battery);
    }
}
