package buildcraft.lib.client.render.font;

import java.util.List;

import net.minecraft.client.Minecraft;
import net.minecraft.client.gui.FontRenderer;
import net.minecraft.client.resources.IResourceManager;
import net.minecraft.item.EnumDyeColor;
import net.minecraft.util.ResourceLocation;
import net.minecraft.util.text.TextFormatting;

import buildcraft.api.core.BCLog;

import buildcraft.lib.misc.ColourUtil;

public class SpecialColourFontRenderer extends FontRenderer {
    public static final SpecialColourFontRenderer INSTANCE = new SpecialColourFontRenderer();

    private SpecialColourFontRenderer() {
        super(Minecraft.func_71410_x().field_71474_y, new ResourceLocation("textures/font/ascii.png"),
            Minecraft.func_71410_x().field_71446_o, false);
    }

    @Override
    public int func_175065_a(String text, float x, float y, int color, boolean dropShadow) {

        int next = text.indexOf(ColourUtil.COLOUR_SPECIAL_START);
        int taken = 0;

        if (next > 0) {
            // Render some of it normally
            x = getRealRenderer().func_175065_a(text.substring(0, next), x, y, color, dropShadow);
            taken = next;
        }

        while (next != -1) {

            int end = text.indexOf(TextFormatting.RESET.toString());
            String sub;
            if (end > 0) {
                sub = text.substring(next, end);
                taken = end;
            } else {
                sub = text.substring(next);
                taken = text.length();
            }

            char c = text.charAt(next + 3);
            int thisColour = color;
            try {
                int ord = Integer.parseInt(Character.toString(c), 16);
                thisColour = ColourUtil.getLightHex(EnumDyeColor.func_176764_b(ord));
            } catch (NumberFormatException nfe) {
                BCLog.logger
                    .warn("[lib.font] Invalid colour string for SpecialColourFontRenderer! " + nfe.getMessage());
            }

            x = getRealRenderer().func_175065_a(sub, x, y, thisColour, dropShadow);

            next = text.indexOf(ColourUtil.COLOUR_SPECIAL_START, end);
        }

        if (taken < text.length()) {
            x = getRealRenderer().func_175065_a(text.substring(taken), x, y, color, dropShadow);
        }

        return (int) x;
    }

    private static FontRenderer getRealRenderer() {
        return Minecraft.func_71410_x().field_71466_p;
    }

    // Delegate methods (To ensure we have the exact same behaviour as the normal font renderer)

    @Override
    public void func_110549_a(IResourceManager resourceManager) {
        // NO-OP
    }

    @Override
    public int func_78256_a(String text) {
        return getRealRenderer().func_78256_a(text);
    }

    @Override
    public int func_78263_a(char character) {
        return getRealRenderer().func_78263_a(character);
    }

    @Override
    public String func_78269_a(String text, int width) {
        return getRealRenderer().func_78269_a(text, width);
    }

    @Override
    public String func_78262_a(String text, int width, boolean reverse) {
        return getRealRenderer().func_78262_a(text, width, reverse);
    }

    @Override
    public int func_78267_b(String str, int maxLength) {
        return getRealRenderer().func_78267_b(str, maxLength);
    }

    @Override
    public void func_78264_a(boolean unicodeFlagIn) {
        getRealRenderer().func_78264_a(unicodeFlagIn);
    }

    @Override
    public boolean func_82883_a() {
        return getRealRenderer().func_82883_a();
    }

    @Override
    public void func_78275_b(boolean bidiFlagIn) {
        getRealRenderer().func_78275_b(bidiFlagIn);
    }

    @Override
    public List<String> func_78271_c(String str, int wrapWidth) {
        return getRealRenderer().func_78271_c(str, wrapWidth);
    }

    @Override
    public boolean func_78260_a() {
        return getRealRenderer().func_78260_a();
    }

    @Override
    public int func_175064_b(char character) {
        return getRealRenderer().func_175064_b(character);
    }
}
